@@ -242,6 +242,7 @@ class IPATransformer(nn.Module):
         dim,
         depth,
         num_tokens = None,
+        predict_points = False,
         **kwargs
     ):
         super().__init__()
@@ -253,13 +254,15 @@ class IPATransformer(nn.Module):
             self.quaternion_to_matrix = quaternion_to_matrix
             self.quaternion_multiply = quaternion_multiply
         except ImportError as err:
-            print('unable to pytorch3d - please install with `conda install pytorch3d -c pytorch3d`')
+            print('unable to import pytorch3d - please install with `conda install pytorch3d -c pytorch3d`')
             raise err
 
-        # modules
+        # embedding
 
         self.token_emb = nn.Embedding(num_tokens, dim) if exists(num_tokens) else None
 
+        # layers
+
         self.layers = nn.ModuleList([])
         for _ in range(depth):
             self.layers.append(nn.ModuleList([
@@ -267,6 +270,13 @@ class IPATransformer(nn.Module):
                 nn.Linear(dim, 6)
             ]))
 
+        # output
+
+        self.predict_points = predict_points
+
+        if predict_points:
+            self.to_points = nn.Linear(dim, 3)
+
     def forward(
         self,
         single_repr,
@@ -310,7 +320,13 @@ class IPATransformer(nn.Module):
             quaternion_update, translation_update = to_update(x).chunk(2, dim = -1)
             quaternion_update = F.pad(quaternion_update, (1, 0), value = 1.)
 
-            quaternions = quaternions + quaternion_multiply(quaternions, quaternion_update)
+            quaternions = quaternion_multiply(quaternions, quaternion_update)
             translations = translations + einsum('b n c, b n c r -> b n r', translation_update, rotations)
 
-        return x, translations, quaternions
+        if not self.predict_points:
+            return x, translations, quaternions
+
+        points_local = self.to_points(x)
+        rotations = quaternion_to_matrix(quaternions)
+        points_global = einsum('b n c, b n c d -> b n d', points_local, rotations) + translations
+        return points_global
